#ifndef CDMemory_H
#define CDMemory_H



#include "CDSys.h"	/* Defines which system/compiler we're running on */


/*
   "CDMemory" memory allocation routines
   Copyright 1991 Capilano Computing Systems Ltd.
   All rights reserved.
   
   Modifications
   
   1/31/91	Added "CD" in front of all names
*/



/* CDAllocates a block of the given size and returns a pointer to it.
   If the block cannot be allocated, a HALT is executed */

pQUALIFIER void *CDAllocate(long Size );



/* Returns the size of the block whose pointer is given, or
   -1 if the pointer does not point to a valid block (it is not
   guaranteed to detect an invalid block).
   
   IMPORTANT: The size returned may be up to 16 bytes larger than
   the amount originally requested.  This is because the originally
   allocated block may have left a few free bytes at the end that
   were insufficient to make another free block.  These are then
   left trailing on this block. */

pQUALIFIER long CDGetAllocSize(void *addr);


/* CDDeallocates the block whose pointer is given */

pQUALIFIER void CDDeallocate(void *addr);


/* Returns TRUE if a block of the given size can be allocated */

pQUALIFIER Boolean CDCanAllocate(long size);


/* CDDeallocates all allocated memory */

pQUALIFIER void CDDeallocateAll(void);


/* Performs a verification on the allocation structures.  If "incremental" is TRUE,
   then each time the routine is called a single superblock is checked.  Each subsequent
   call moves to the next block.  This will reduce execution time so that it can be
   called continuously while a program is idling.  If any error is encountered,
   a pointer to the invalid block is returned, else NIL is returned */

pQUALIFIER void *CDAllocVerify(Boolean incremental);


/* Sets up a map of memory allocation.  If "sizeOnly" is TRUE, it returns the size in
   bytes of the table that will be created but does not set up the table.  If "sizeOnly"
   is FALSE it assumes that "tablePtr" points to a memory area of sufficient size
   to contain the table.  Table is of the following form:

   blockAddress (4 bytes) blockLength (4 bytes)
*/

pQUALIFIER void CDAllocMap(void *tableAddr, long *tableSize, Boolean sizeOnly);




#define cCDDeallocFill 0
#define cCDDeallocSADE 1
#define cCDDeallocDebug 2
#define cCDLowMemCheck 3

/* "CDAllocTestMode" enables or disables memory allocation test mode.
   When enabled, all unallocated areas are filled with nasty odd numbers
   that will cause a bomb if used as pointers.

   Bit 0 - Fill unused blocks with bad values.
   Bit 1 - Trap to SADE on bad deallocate
   Bit 2 - Debugger trap on bad deallocate
   Bit 3 - Do checksum on low memory when AllocVerify called
*/

pQUALIFIER void CDAllocTestMode(short testMode);



/* "CDSetMemErrorProc" passes a pointer to a procedure
   to call if a "CDAllocate" is about to fail for lack of room.
   The given procedure is called with a long argument specifying
   the size of block needed.  If possible, the error proc should
   free flexible objects to make this amount of room available. */

typedef pascal long (*tMemErrorProc)(long);

pQUALIFIER void CDSetMemErrorProc(tMemErrorProc memProc);



/* One of the following routines must be called before any of the above */

pQUALIFIER void CDMemory_Init(void);


pQUALIFIER void CDMemoryInit(void);




#endif /*CDMemory_H*/

/* End */
